#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wmul_click_utils']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['click']

extras_require = \
{'test': ['pytest', 'wmul_test_utils'], 'tox': ['tox']}

setup(name='wmul_click_utils',
      version='0.1.0',
      description="Additional utilities for use with Python's click cli library.",
      author=None,
      author_email='Michael Stanley <stanley50@marshall.edu>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
